#ifndef INITIALIZATION_MESSAGE_H
#define INITIALIZATION_MESSAGE_H

// Copyright (c) Clifton Labs, Inc.
// All rights reserved.

// CLIFTON LABS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
// SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
// NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  CLIFTON LABS SHALL NOT BE
// LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, RESULT
// OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

// By using or copying this Software, Licensee agrees to abide by the
// intellectual property laws, and all other applicable laws of the
// U.S., and the terms of this license.

// Authors:
//          Dale E. Martin              dmartin@cliftonlabs.com

/**
   This is the class defining the termination token that gets passed
   around.  Essentially, it has a color and a terminator.  

   The color can be WHITE, which means it's on the first cycle,
   RED, which means it's on it's second cycle, or BLACK.
*/

class SerializedInstance;

#include "KernelMessage.h"

/**
   This class represents the token that gets passed between
   communication managers at initialization time.
*/

class InitializationMessage : public KernelMessage {
public:
  InitializationMessage( unsigned int source,
			 unsigned int dest,
			 const vector<string> &initObjectNames,
			 unsigned int initNumSimulationManagers ) :
    KernelMessage( source, dest ),
    objectNames( initObjectNames ),
    numSimulationManagers( initNumSimulationManagers ){}
  
  void serialize( SerializedInstance * ) const;
  static Serializable *deserialize( SerializedInstance *data );

  static const string &getInitializationMessageType();

  const string &getDataType() const {
    return getInitializationMessageType();
  }

  static void registerDeserializer();
  
  const vector<string> &getObjectNames() const { return objectNames; }
  const unsigned &getNumSimulationManagers() const { return numSimulationManagers; }
  
private:
  static const vector<string> deserializeStringVector( SerializedInstance *data );

  const vector<string> objectNames;
  const unsigned numSimulationManagers;

};

#endif
